/*******************************************************************
  Interfacing PIC18F46K22 MCU with SSD1306 OLED display (SPI mode).
  C Code for mikroC PRO for PIC compiler.
  Internal oscillator used @ 16MHz
  Configuration words: CONFIG1H = 0x0028
                       CONFIG2L = 0x0018
                       CONFIG2H = 0x003C
                       CONFIG3H = 0x0037
                       CONFIG4L = 0x0081
                       CONFIG5L = 0x000F
                       CONFIG5H = 0x00C0
                       CONFIG6L = 0x000F
                       CONFIG6H = 0x00E0
                       CONFIG7L = 0x000F
                       CONFIG7H = 0x0040
  This is a free software with NO WARRANTY.
  https://simple-circuit.com/

*********************************************************************
This is an example for our Monochrome OLEDs based on SSD1306 drivers

  Pick one up today in the adafruit shop!
  ------> http://www.adafruit.com/category/63_98

This example is for a 128x64 size display using SPI to communicate
4 or 5 pins are required to interface

Adafruit invests time and resources providing this open source code,
please support Adafruit and open-source hardware by purchasing
products from Adafruit!

Written by Limor Fried/Ladyada  for Adafruit Industries.
BSD license, check license.txt for more information
All text above, and the splash screen must be included in any redistribution
****************************************************************************/

// use SSD1306 OLED display with mikroC hardware SPI1 module
#define SSD1306_HARD_SPI1

// define other pins: RST, DC and CS
#define SSD1306_RST       RD4_bit
#define SSD1306_DC        RD5_bit
#define SSD1306_CS        RD6_bit
#define SSD1306_RST_DIR   TRISD4_bit
#define SSD1306_DC_DIR    TRISD5_bit
#define SSD1306_CS_DIR    TRISD6_bit

#include "SSD1306.c"      // include SSD1306 OLED display driver source code
#include "GFX_Library.c"  // include graphics library source code

#define NUMFLAKES 10
#define XPOS 0
#define YPOS 1
#define DELTAY 2

#define LOGO16_GLCD_HEIGHT 16
#define LOGO16_GLCD_WIDTH  16
const char logo16_glcd_bmp[] =
{ 0b00000000, 0b11000000,
  0b00000001, 0b11000000,
  0b00000001, 0b11000000,
  0b00000011, 0b11100000,
  0b11110011, 0b11100000,
  0b11111110, 0b11111000,
  0b01111110, 0b11111111,
  0b00110011, 0b10011111,
  0b00011111, 0b11111100,
  0b00001101, 0b01110000,
  0b00011011, 0b10100000,
  0b00111111, 0b11100000,
  0b00111111, 0b11110000,
  0b01111100, 0b11110000,
  0b01110000, 0b01110000,
  0b00000000, 0b00110000 };


void testdrawbitmap(const uint8_t *bitmap, uint8_t w, uint8_t h) {
  static uint8_t icons[NUMFLAKES][3], f, x_pos, y_pos;

  // initialize
  for (f=0; f< NUMFLAKES; f++) {
    icons[f][XPOS] = rand();
    icons[f][YPOS] = 0;
    icons[f][DELTAY] = (rand() % 5) + 1;
  }

  while (1) {
    uint8_t f;
    // draw each icon
    for (f=0; f< NUMFLAKES; f++) {
      x_pos = icons[f][XPOS];
      y_pos = icons[f][YPOS];
      display_drawBitmapV2(x_pos, y_pos, bitmap, w, h, WHITE);
    }
    display();
    delay_ms(200);

    // then erase it + move it
    for (f=0; f< NUMFLAKES; f++) {
      x_pos = icons[f][XPOS];
      y_pos = icons[f][YPOS];
      display_drawBitmapV2(x_pos, y_pos, bitmap, w, h, BLACK);
      // move it
      icons[f][YPOS] += icons[f][DELTAY];
      // if its gone, reinit
      if (icons[f][YPOS] > display_height) {
        icons[f][XPOS] = rand();
        icons[f][YPOS] = 0;
        icons[f][DELTAY] = (rand() % 5) + 1;
      }
    }
   }
}

void testdrawchar(void) {
  uint8_t i;
  display_setTextSize(1);
  display_setTextColor(WHITE, WHITE);
  display_setCursor(0, 0);

  for (i=0; i < 170; i++) {
    if (i == '\n' || i == '\r') continue;
    display_putc(i);
  }
  display();
  delay_ms(1);
}

void testdrawcircle(void) {
  uint16_t i;
  for (i=0; i<display_height; i+=2) {
    display_drawCircle(display_width/2, display_height/2, i, WHITE);
    display();
    delay_ms(1);
  }
}

void testfillrect(void) {
  uint8_t color = 1;
  uint16_t i;
  for (i=0; i<display_height/2; i+=3) {
    // alternate colors
    display_fillRect(i, i, display_width-i*2, display_height-i*2, color%2);
    display();
    delay_ms(1);
    color++;
  }
}

void testdrawtriangle(void) {
  uint16_t i;
  for (i=0; i<min(display_width,display_height)/2; i+=5) {
    display_drawTriangle(display_width/2, display_height/2-i,
                     display_width/2-i, display_height/2+i,
                     display_width/2+i, display_height/2+i, WHITE);
    display();
    delay_ms(1);
  }
}

void testfilltriangle(void) {
  uint8_t color = WHITE;
  int16_t i;
  for (i =0 ; i < min(display_width,display_height)/2; i += 5) {
    display_fillTriangle(display_width/2, display_height/2-i,
                     display_width/2-i, display_height/2+i,
                     display_width/2+i, display_height/2+i, WHITE);
    if (color == WHITE) color = BLACK;
    else color = WHITE;
    display();
    delay_ms(1);
  }
}

void testdrawroundrect(void) {
  uint16_t i;
  for (i=0; i<display_height/2-2; i+=2) {
    display_drawRoundRect(i, i, display_width-2*i, display_height-2*i, display_height/4, WHITE);
    display();
    delay_ms(1);
  }
}

void testfillroundrect(void) {
  uint8_t color = WHITE;
  uint16_t i;
  for (i=0; i<display_height/2-2; i+=2) {
    display_fillRoundRect(i, i, display_width-2*i, display_height-2*i, display_height/4, color);
    if (color == WHITE) color = BLACK;
    else color = WHITE;
    display();
    delay_ms(1);
  }
}

void testdrawrect(void) {
  uint16_t i;
  for (i=0; i<display_height/2; i+=2) {
    display_drawRect(i, i, display_width-2*i, display_height-2*i, WHITE);
    display();
    delay_ms(1);
  }
}

void testdrawline() {
  int16_t i;
  for (i=0; i<display_width; i+=4) {
    display_drawLine(0, 0, i, display_height-1, WHITE);
    display();
    delay_ms(1);
  }
  for (i=0; i<display_height; i+=4) {
    display_drawLine(0, 0, display_width-1, i, WHITE);
    display();
    delay_ms(1);
  }
  delay_ms(250);

  display_clear();
  for (i=0; i<display_width; i+=4) {
    display_drawLine(0, display_height-1, i, 0, WHITE);
    display();
    delay_ms(1);
  }
  for (i=display_height-1; i>=0; i-=4) {
    display_drawLine(0, display_height-1, display_width-1, i, WHITE);
    display();
    delay_ms(1);
  }
  delay_ms(250);

  display_clear();
  for (i=display_width-1; i>=0; i-=4) {
    display_drawLine(display_width-1, display_height-1, i, 0, WHITE);
    display();
    delay_ms(1);
  }
  for (i=display_height-1; i>=0; i-=4) {
    display_drawLine(display_width-1, display_height-1, 0, i, WHITE);
    display();
    delay_ms(1);
  }
  delay_ms(250);

  display_clear();
  for (i=0; i<display_height; i+=4) {
    display_drawLine(display_width-1, 0, 0, i, WHITE);
    display();
    delay_ms(1);
  }
  for (i=0; i<display_width; i+=4) {
    display_drawLine(display_width-1, 0, i, display_height-1, WHITE);
    display();
    delay_ms(1);
  }
  delay_ms(250);
}

void testscrolltext(void) {
  display_setTextSize(2);
  display_setTextColor(WHITE, WHITE);
  display_setCursor(10,0);
  display_clear();
  display_puts("scroll\r\n");
  display();
  delay_ms(1);

  display_startscrollright(0x00, 0x0F);
  delay_ms(2000);
  display_stopscroll();
  delay_ms(1000);
  display_startscrollleft(0x00, 0x0F);
  delay_ms(2000);
  display_stopscroll();
  delay_ms(1000);
  display_startscrolldiagright(0x00, 0x07);
  delay_ms(2000);
  display_startscrolldiagleft(0x00, 0x07);
  delay_ms(2000);
  display_stopscroll();
}

// main function
void main()
{
  OSCCON = 0x70;   // set internal oscillator to 16MHz
  ANSELC = 0;      // configure all PORTC pins as digital
  ANSELD = 0;      // configure all PORTD pins as digital
  SPI1_Init();     // initialize the SPI1 module with default settings

  // by default, we'll generate the high voltage from the 3.3v line internally! (neat!)
  // initialize the SSD1306 OLED display
  SSD1306_begin(SSD1306_SWITCHCAPVCC);
  // init done

  // show image buffer on the display hardware.
  // Since the buffer is intialized with an Adafruit splashscreen
  // internally, this will display the splashscreen.
  display();
  delay_ms(2000);

  // clear the buffer.
  display_clear();

  // draw a single pixel
  display_drawPixel(10, 10, WHITE);
  // Show the display buffer on the hardware.
  // NOTE: You _must_ call display after making any drawing commands
  // to make them visible on the display hardware!
  display();
  delay_ms(2000);
  display_clear();

  // draw many lines
  testdrawline();
  display();
  delay_ms(2000);
  display_clear();

  // draw rectangles
  testdrawrect();
  display();
  delay_ms(2000);
  display_clear();

  // draw multiple rectangles
  testfillrect();
  display();
  delay_ms(2000);
  display_clear();

  // draw mulitple circles
  testdrawcircle();
  display();
  delay_ms(2000);
  display_clear();

  // draw a white circle, 10 pixel radius
  display_fillCircle(display_width/2, display_height/2, 10, WHITE);
  display();
  delay_ms(2000);
  display_clear();

  testdrawroundrect();
  delay_ms(2000);
  display_clear();

  testfillroundrect();
  delay_ms(2000);
  display_clear();

  testdrawtriangle();
  delay_ms(2000);
  display_clear();

  testfilltriangle();
  delay_ms(2000);
  display_clear();

  // draw the first ~12 characters in the font
  testdrawchar();
  display();
  delay_ms(2000);
  display_clear();

  // draw scrolling text
  testscrolltext();
  delay_ms(2000);
  display_clear();

  // text display tests
  display_setTextSize(1);
  display_setTextColor(WHITE, BLACK);
  display_setCursor(0, 0);
  display_printf("Hello, world!\r\n");
  display_setTextColor(BLACK, WHITE); // 'inverted' text
  display_printf("%4.2f\r\n", 3.141592);
  display_setTextSize(2);
  display_setTextColor(WHITE, BLACK);
  display_printf("0x%LX\r\n", 0xDEADBEEF);
  display();
  delay_ms(2000);
  display_clear();

  // miniature bitmap display
  display_drawBitmapV2(30, 16,  logo16_glcd_bmp, 16, 16, WHITE);
  display();
  delay_ms(1);

  // invert the display
  display_invert(true);
  delay_ms(1000);
  display_invert(false);
  delay_ms(1000);
  display_clear();

  // draw a bitmap icon and 'animate' movement
  testdrawbitmap(logo16_glcd_bmp, LOGO16_GLCD_HEIGHT, LOGO16_GLCD_WIDTH);
}

// end of code.